//Exports to ISO-8859-15, CR/LF for maximum Microsoft Windows/Excel compatibility
import haxe.crypto.Base64;
import haxe.crypto.Crc32;
import haxe.io.Bytes;
import haxe.io.BytesOutput;
import haxe.zip.Entry;
import haxe.zip.Writer;
using StringTools;

class Export {
	static function assemble (parts : Iterable<String>, separator : String) : String {
		var str = "";
		for (part in parts) {
			if (part.indexOf(separator) == -1) {
				str += part + separator;
			} else {
				str += "\"" + part + "\"" + separator;
			}
		}
		return str.substring(0, str.length - 1);
	}

	public static function getEncodedZip(promo : Promotion) : String {
		var files : List<Entry> = new List();
		files.add(getEntry("eleves.csv", getEleves(promo)));
		files.add(getEntry("specialites.csv", getSpecialite(promo)));
		files.add(getEntry("nonAffectes.csv", getNonAffectes(promo)));
		files.add(getEntry("secondsChoix.csv", getSecondsChoix(promo)));
		for (i in 0...promo.alignements.length) {
			for (regspe in promo.alignements[i]) {
				for (grp in regspe.groupes) {
					var filename = Std.string(i + 1) + "-" + strToFilename(grp.libelle) + ".csv";
					files.add(getEntry(filename, getGroupe(grp)));
				}
			}
		}

		var output = new BytesOutput();
		var zipWriter = new Writer(output);
		zipWriter.write(files);
		return Base64.encode(output.getBytes());
	}

	static function strToFilename(str : String) : String {
		str = str.toLowerCase();
		for (accent in ["é", "è", "ê", "ë"]) str = str.replace(accent, "e");
		for (accent in ["à", "â"]) str = str.replace(accent, "a");
		for (accent in ["î", "ï"]) str = str.replace(accent, "i");
		for (accent in ["ù", "û"]) str = str.replace(accent, "u");
		str = str.replace("ô", "o");
		str = str.replace(" ", "-");
		for (specialChar in [",", ";", ":", "&", "$", "!", "?", ".", "@", "~", "(", ")", "[", "]", "{", "}"]) str = str.replace(specialChar, "");
		str = str.replace("--", "-");
		return str;
	}

	static function getISOBytes(str : String) : Bytes {
		var result = Bytes.alloc(str.length);
		for (i in 0...str.length) {
			result.set(i, str.charCodeAt(i));
		}
		return result;
	}

	static function getEntry(filename : String, data : String) : Entry {
		//~ var bytes = Bytes.ofString(data); //UTF-8
		var bytes = getISOBytes(data);
		return {
			fileName : filename,
			fileSize : bytes.length,
			fileTime : Date.now(),
			compressed : false,
			dataSize : 0,
			data : bytes,
			crc32 : Crc32.make(bytes)
		};
	}

	static function getEleves(promo : Promotion) : String {
		var str = "";
		for (elv in promo.eleves) {
			str += assemble(Lambda.concat([ elv.nom, elv.prenom ], elv.voeuxSaisis), ";") + "\r\n";
		}
		return str;
	}

	static function getGroupe(grp : Groupe) : String {
		var str = "";
		for (elv in grp.elevesAffectes) {
			str += assemble(Lambda.concat([ elv.nom, elv.prenom ], elv.voeuxSaisis), ";") + "\r\n";
		}
		return str;
	}

	static function getSpecialite(promo : Promotion) : String {
		var str = "Libellé;Effectif;Nb groupes classe;Nb groupes réduits;Capacité;Heures\r\n";
		for (spe in promo.specialites) {
			str += assemble([ spe.libelle, Std.string(spe.effectif),
				Std.string(spe.nbGroupesClasse), Std.string(spe.nbGroupesReduits), Std.string(spe.capacite),
				Std.string(spe.nbGroupesClasse * spe.coutEffectifClasse + spe.nbGroupesReduits * spe.coutEffectifReduit) ], ";") + "\r\n";
		}
		return str;
	}

	static function getNonAffectes(promo : Promotion) : String {
		var str = "";
		for (elv in promo.nonAffectes) {
			str += assemble(Lambda.concat([ elv.nom, elv.prenom ], elv.voeuxSaisis), ";") + "\r\n";
		}
		return str;
	}

	static function getSecondsChoix(promo : Promotion) : String {
		var str = "";
		for (elv in promo.affectesSecondChoix) {
			str += assemble(Lambda.concat([ elv.nom, elv.prenom ], elv.voeuxSaisis), ";") + "\r\n";
		}
		return str;
	}
}
